@@ -23,19 +23,13 @@
 #endif /* HAVE_CONFIG_H */
 
 #include <string.h>
-#include <stdio.h>
 #include <glib-object.h>
 #include <glib/gi18n.h>
-#include <glib/gerror.h>
-#include <glib/glist.h>
-#include <glib/gstrfuncs.h>
-#include <glib/gmessages.h>
+#include <glib.h>
 #include <gconf/gconf-client.h>
 #include <musicbrainz/queries.h>
 #include <musicbrainz/mb_c.h>
-#include <nautilus-burn.h>
 #include <stdlib.h>
-#include <unistd.h>
 
 #include "sj-metadata-musicbrainz.h"
 #include "sj-structures.h"
@@ -55,9 +49,7 @@ struct SjMetadataMusicbrainzPrivate {
   char *http_proxy;
   int http_proxy_port;
   char *cdrom;
-  /* TODO: remove and use an async queue? */
   GList *albums;
-  GError *error;
 };
 
 #define GET_PRIVATE(o)  \
@@ -93,54 +85,6 @@ get_duration_from_sectors (int sectors)
   return (sectors * BYTES_PER_SECTOR / BYTES_PER_SECOND);
 }
 
-static GList*
-get_offline_track_listing(SjMetadata *metadata, GError **error)
-{
-  SjMetadataMusicbrainzPrivate *priv;
-  GList* list = NULL;
-  AlbumDetails *album;
-  TrackDetails *track;
-  int num_tracks, i;
-
-  g_return_val_if_fail (metadata != NULL, NULL);
-  priv = SJ_METADATA_MUSICBRAINZ (metadata)->priv;
-
-  if (!mb_Query (priv->mb, MBQ_GetCDTOC)) {
-    char message[255];
-    mb_GetQueryError (priv->mb, message, 255);
-    g_set_error (error,
-                 SJ_ERROR, SJ_ERROR_CD_LOOKUP_ERROR,
-                 _("Cannot read CD: %s"), message);
-    return NULL;
-  }
-  
-  num_tracks = mb_GetResultInt (priv->mb, MBE_TOCGetLastTrack);
-
-  album = g_new0 (AlbumDetails, 1);
-  album->artist = g_strdup (_("Unknown Artist"));
-  album->title = g_strdup (_("Unknown Title"));
-  album->genre = NULL;
-  for (i = 1; i <= num_tracks; i++) {
-    track = g_new0 (TrackDetails, 1);
-    track->album = album;
-    track->number = i;
-    track->title = g_strdup_printf (_("Track %d"), i);
-    track->artist = g_strdup (album->artist);
-    track->duration = get_duration_from_sectors (mb_GetResultInt1 (priv->mb, MBE_TOCGetTrackNumSectors, i+1));
-    album->tracks = g_list_append (album->tracks, track);
-    album->number++;
-  }
-  return g_list_append (list, album);
-}
-
-static gboolean
-fire_signal_idle (SjMetadataMusicbrainz *m)
-{
-  g_return_val_if_fail (SJ_IS_METADATA_MUSICBRAINZ (m), FALSE);
-  g_signal_emit_by_name (G_OBJECT (m), "metadata", m->priv->albums, m->priv->error);
-  return FALSE;
-}
-
 /**
  * Virtual methods
  */
@@ -323,63 +267,37 @@ convert_encoding(char **str)
   g_free (iso8859);
 }
 
-static gpointer
-lookup_cd (SjMetadata *metadata)
+static GList *
+mb_list_albums (SjMetadata *metadata, char **url, GError **error)
 {
   /** The size of the buffer used in MusicBrainz lookups */
   SjMetadataMusicbrainzPrivate *priv;
   GList *albums = NULL;
   GList *al, *tl;
-  char data[256];
+  char data[1024];
   int num_albums, i, j;
-  NautilusBurnMediaType type;
-  NautilusBurnDriveMonitor *monitor;
-  NautilusBurnDrive *drive;
 
-  /* TODO: fire error signal */
   g_return_val_if_fail (metadata != NULL, NULL);
   g_return_val_if_fail (SJ_IS_METADATA_MUSICBRAINZ (metadata), NULL);
   priv = SJ_METADATA_MUSICBRAINZ (metadata)->priv;
   g_return_val_if_fail (priv->cdrom != NULL, NULL);
-  priv->error = NULL; /* TODO: hack */
-
-  if (! nautilus_burn_initialized ()) {
-    nautilus_burn_init ();
-  }
-  monitor = nautilus_burn_get_drive_monitor ();
-  drive = nautilus_burn_drive_monitor_get_drive_for_device (monitor, priv->cdrom);
-  if (drive == NULL) {
-    return NULL;
-  }
-  type = nautilus_burn_drive_get_media_type (drive);
-  nautilus_burn_drive_unref (drive);
-
-  if (type == NAUTILUS_BURN_MEDIA_TYPE_ERROR) {
-    char *msg;
-    SjError err;
-
-    if (access (priv->cdrom, W_OK) == 0) {
-      msg = g_strdup_printf (_("Device '%s' does not contain any media"), priv->cdrom);
-      err = SJ_ERROR_CD_NO_MEDIA;
-    } else {
-      msg = g_strdup_printf (_("Device '%s' could not be opened. Check the access permissions on the device."), priv->cdrom);
-      err = SJ_ERROR_CD_PERMISSION_ERROR;
-    }
-    priv->error = g_error_new (SJ_ERROR, err, _("Cannot read CD: %s"), msg);
-    g_free (msg);
 
+  if (sj_metadata_helper_check_media (priv->cdrom, error) == FALSE) {
     priv->albums = NULL;
-    g_idle_add ((GSourceFunc)fire_signal_idle, metadata);
     return NULL;
   }
 
   get_rdf (metadata);
 
+  if (url != NULL) {
+    mb_GetWebSubmitURL(priv->mb, data, sizeof(data));
+    *url = g_strdup(data);
+  }
+
   num_albums = mb_GetResultInt(priv->mb, MBE_GetNumAlbums);
   if (num_albums < 1) {
-    priv->albums = get_offline_track_listing (metadata, &(priv->error));
-    g_idle_add ((GSourceFunc)fire_signal_idle, metadata);
-    return priv->albums;
+    priv->albums = NULL;
+    return NULL;
   }
 
   for (i = 1; i <= num_albums; i++) {
@@ -410,28 +328,33 @@ lookup_cd (SjMetadata *metadata)
           album->artist = g_strdup (_("Unknown Artist"));
         }
       }
-      if (data && mb_GetResultData(priv->mb, MBE_AlbumGetAlbumArtistSortName, data, sizeof (data))) {
+      if (mb_GetResultData(priv->mb, MBE_AlbumGetAlbumArtistSortName, data, sizeof (data))) {
         album->artist_sortname = g_strdup (data);
       }
     }
 
     if (mb_GetResultData(priv->mb, MBE_AlbumGetAlbumName, data, sizeof (data))) {
-      album->title = g_strdup (data);
+      char *new_title;
+      new_title = sj_metadata_helper_scan_disc_number (data, &album->disc_number);
+      if (new_title)
+        album->title = new_title;
+      else
+        album->title = g_strdup (data);
     } else {
       album->title = g_strdup (_("Unknown Title"));
     }
 
+    if (mb_GetResultData(priv->mb, MBE_AlbumGetAmazonAsin, data, sizeof (data))) {
+      album->asin = g_strdup (data);
+    }
+
     {
       int num_releases;
       num_releases = mb_GetResultInt (priv->mb, MBE_AlbumGetNumReleaseDates);
       if (num_releases > 0) {
         mb_Select1(priv->mb, MBS_SelectReleaseDate, 1);
         if (mb_GetResultData(priv->mb, MBE_ReleaseGetDate, data, sizeof (data))) {
-          int matched, year=1, month=1, day=1;
-          matched = sscanf(data, "%u-%u-%u", &year, &month, &day);
-          if (matched >= 1) {
-            album->release_date = g_date_new_dmy ((day == 0) ? 1 : day, (month == 0) ? 1 : month, year);
-          }
+          album->release_date = sj_metadata_helper_scan_date (data);
         }
         mb_Select(priv->mb, MBS_Back);
       }
@@ -444,9 +367,8 @@ lookup_cd (SjMetadata *metadata)
       g_free (album->title);
       g_free (album);
       g_warning (_("Incomplete metadata for this CD"));
-      priv->albums = get_offline_track_listing (metadata, &(priv->error));
-      g_idle_add ((GSourceFunc)fire_signal_idle, metadata);
-      return priv->albums;
+      priv->albums = NULL;
+      return NULL;
     }
 
     for (j = 1; j <= num_tracks; j++) {
@@ -538,45 +460,9 @@ lookup_cd (SjMetadata *metadata)
     }
   }
 
-  priv->albums = albums;
-  g_idle_add ((GSourceFunc)fire_signal_idle, metadata);
   return albums;
 }
 
-static void
-mb_list_albums (SjMetadata *metadata, GError **error)
-{
-  GThread *thread;
-
-  g_return_if_fail (SJ_IS_METADATA_MUSICBRAINZ (metadata));
-
-  thread = g_thread_create ((GThreadFunc)lookup_cd, metadata, TRUE, error);
-  if (thread == NULL) {
-    g_set_error (error,
-                 SJ_ERROR, SJ_ERROR_INTERNAL_ERROR,
-                 _("Could not create CD lookup thread"));
-    return;
-  }
-}
-
-static char *
-mb_get_submit_url (SjMetadata *metadata)
-{
-  SjMetadataMusicbrainzPrivate *priv;
-  char url[1025];
-
-  g_return_val_if_fail (metadata != NULL, NULL);
-
-  priv = SJ_METADATA_MUSICBRAINZ (metadata)->priv;
-
-  if (mb_GetWebSubmitURL(priv->mb, url, 1024)) {
-    return g_strdup(url);
-  } else {
-    return NULL;
-  }
-}
-
-
 /*
  * GObject methods
  */
@@ -586,7 +472,6 @@ metadata_interface_init (gpointer g_iface, gpointer iface_data)
 {
   SjMetadataClass *klass = (SjMetadataClass*)g_iface;
   klass->list_albums = mb_list_albums;
-  klass->get_submit_url = mb_get_submit_url;
 }
 
 static void
@@ -727,5 +612,5 @@ sj_metadata_musicbrainz_class_init (SjMetadataMusicbrainzClass *class)
 GObject *
 sj_metadata_musicbrainz_new (void)
 {
-  return g_object_new (sj_metadata_musicbrainz_get_type (), NULL);
+  return g_object_new (SJ_TYPE_METADATA_MUSICBRAINZ, NULL);
 }
